@@ -15,11 +15,11 @@ sub array_indices {
 		[16, 32, 64, 128, 256], $default);
 
 #	unshift(@$properties, SCP_PUBLIC, MIX_IMPLICIT); # for _base version
-	my $symbols = $self->enumerated_values($prefix, undef, [], @_);
+	my $symbols = $self->enumerated_values($prefix, [], undef, @_);
 	$self->application('constant_function', $properties, @$symbols);
 
 	return($symbols);
 }
 
 
-1;
\ No newline at end of file
+1;
